package com.qmth.themis.admin.api;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.TencentYunUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.MD5Util;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * @Description: 视频源回调接口
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/3/25
 */
@Api(tags = "视频源回调接口")
@RestController
@RequestMapping("/${prefix.url.notify}/monitor/record")
public class TENotifyController {
    private final static Logger log = LoggerFactory.getLogger(TENotifyController.class);

    @Resource
    TOeExamRecordService tOeExamRecordService;

    @Resource
    TencentYunUtil tencentYunUtil;

    @Resource
    RedisUtil redisUtil;

    @ApiOperation(value = "腾讯云视频接口回调")
    @RequestMapping(value = "/tencent", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "结果信息")})
    public Result tencentCallback(@ApiParam(value = "腾讯云视频接口回调信息", required = true) @RequestBody JSONObject jsonObject) throws NoSuchAlgorithmException {
        log.info("腾讯云视频接口回调 jsonObject:{}", jsonObject.toJSONString());
        String callbackPwd = tencentYunUtil.getTencentYunDomain().getCallbackPwd();
        Optional.ofNullable(callbackPwd).orElseThrow(() -> new BusinessException("配置文件回调密码为空"));

        String sign = (String) jsonObject.get("sign");
        Optional.ofNullable(sign).orElseThrow(() -> new BusinessException("腾讯云签名为空"));

        Long t = Long.parseLong(String.valueOf(jsonObject.get("t")));
        Optional.ofNullable(t).orElseThrow(() -> new BusinessException("腾讯云回调时间为空"));

        if (!MD5Util.verify(callbackPwd + String.valueOf(t), sign)) {
            throw new BusinessException("腾讯云签名不匹配");
        }

        String eventType = String.valueOf(jsonObject.get("event_type"));
        //说明是录制视频回调成功
        if (Objects.nonNull(eventType) && Objects.equals(eventType, "100")) {
            String streamId = String.valueOf(jsonObject.get("stream_id"));
            Long recordId = null;
            if (Objects.nonNull(streamId)) {
                String[] str = streamId.split("_");
                recordId = Long.parseLong(str[2]);
                String tencentVideoUrl = ExamRecordCacheUtil.getTencentVideoUrl(recordId);
                if (Objects.nonNull(tencentVideoUrl)) {//说明缓存还在
                    JSONArray jsonArrayDb = JSONArray.parseArray(tencentVideoUrl);
                    int count = jsonArrayDb.size();
                    JSONArray jsonArray = this.getVideoUrl(tencentVideoUrl, streamId, jsonObject);
                    if (Objects.nonNull(jsonArray) && jsonArray.size() > 0 && count != jsonArray.size()) {
                        ExamRecordCacheUtil.setTencentVideoUrl(recordId, jsonArray.toJSONString());
                        tOeExamRecordService.sendExamRecordDataSaveMq(recordId, System.currentTimeMillis());
                    }
                } else {
                    TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(recordId);
                    int count = 0;
                    JSONArray jsonArrayDb = null;
                    if (Objects.nonNull(tOeExamRecord.getTencentVideoUrl())) {
                        jsonArrayDb = JSONArray.parseArray(tOeExamRecord.getTencentVideoUrl());
                        count = jsonArrayDb.size();
                    }
                    JSONArray jsonArray = this.getVideoUrl(tOeExamRecord.getTencentVideoUrl(), streamId, jsonObject);
                    if (Objects.nonNull(jsonArray) && jsonArray.size() > 0 && count != jsonArray.size()) {
                        tOeExamRecord.setTencentVideoUrl(jsonArray.toJSONString());
                        tOeExamRecordService.updateById(tOeExamRecord);
                    }
                }
            }
        }
        return ResultUtil.ok(true);
    }

    /**
     * 获取视频源地址
     *
     * @param tencentVideoUrl
     * @param streamId
     * @param jsonObject
     * @return
     */
    private JSONArray getVideoUrl(String tencentVideoUrl, String streamId, JSONObject jsonObject) {
        Map<String, String> map = new HashMap<>();
        JSONArray jsonArray = null;
        if (Objects.isNull(tencentVideoUrl)) {
            jsonArray = new JSONArray();
        } else {
            jsonArray = JSONArray.parseArray(tencentVideoUrl);
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject json = (JSONObject) jsonArray.get(i);
            String videoSource = SystemConstant.getMonitorRecordStreamId((String) json.get(SystemConstant.VIDEO_SOURCE));
            map.put(videoSource, videoSource);
        }
        String videoSource = SystemConstant.getMonitorRecordStreamId(streamId);
        boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_TENCENT_VIDEO_PREFIX + videoSource, SystemConstant.REDIS_LOCK_TENCENT_VIDEO_TIME_OUT);
        if (lock && Objects.isNull(map.get(videoSource))) {
            try {
                String videoUrl = String.valueOf(jsonObject.get("video_url"));
                JSONObject json = new JSONObject();
                json.put(SystemConstant.VIDEO_SOURCE, videoSource);
                json.put(SystemConstant.VIDEO_URL, videoUrl);
                jsonArray.add(json);
            } finally {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_TENCENT_VIDEO_PREFIX + videoSource);
            }
        }
        return jsonArray;
    }
}
