package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TEExamBreachLogMapper;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.entity.TEExamBreachLog;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.BreachCancelTypeEnum;
import com.qmth.themis.business.enums.BreachTypeEnum;
import com.qmth.themis.business.service.TEExamBreachLogService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 考生违纪处理 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamBreachLogServiceImpl extends ServiceImpl<TEExamBreachLogMapper, TEExamBreachLog> implements TEExamBreachLogService {

    @Resource
    TOeExamRecordService tOeExamRecordService;

    /**
     * 违纪处理
     *
     * @param mapParameter
     */
    @Override
    @Transactional
    public void breachOper(Map<String, Object> mapParameter) {
        Integer status = Integer.parseInt(String.valueOf(mapParameter.get(SystemConstant.STATUS)));
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        List<String> recordIdList = (List<String>) mapParameter.get(SystemConstant.EXAM_RECORD_ID);
        List<TEExamBreachLog> teExamBreachLogList = new ArrayList<>();
        if (status == 0) {//新建违纪
            BreachTypeEnum type = BreachTypeEnum.valueOf(String.valueOf(mapParameter.get(SystemConstant.TYPE)));
            List<TEExamBreachLog> finalTeExamBreachLogList = teExamBreachLogList;
            for (int i = 0; i < recordIdList.size(); i++) {
                Long l = Long.parseLong(String.valueOf(recordIdList.get(i)));
                Long examId = ExamRecordCacheUtil.getExamId(l);
                Long examStudentId = null;
                Long examActivityId = null;
                if (Objects.isNull(examId)) {
                    TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(l);
                    examId = tOeExamRecord.getExamId();
                    examStudentId = tOeExamRecord.getExamStudentId();
                    examActivityId = tOeExamRecord.getExamActivityId();
                } else {
                    examId = ExamRecordCacheUtil.getExamId(l);
                    examStudentId = ExamRecordCacheUtil.getExamStudentId(l);
                    examActivityId = ExamRecordCacheUtil.getExamActivityId(l);
                }
                TEExamBreachLog teExamBreachLog = new TEExamBreachLog(examId, examActivityId, l, examStudentId, type.name(), String.valueOf(mapParameter.get("description")), status);
                teExamBreachLog.setCreateId(tbUser.getId());
                finalTeExamBreachLogList.add(teExamBreachLog);
                ExamRecordCacheUtil.setBreachStatus(l, 0);
                tOeExamRecordService.sendExamRecordDataSaveMq(l, System.currentTimeMillis());
            }
        } else {//撤销违纪
            BreachCancelTypeEnum type = BreachCancelTypeEnum.valueOf(String.valueOf(mapParameter.get(SystemConstant.TYPE)));
            QueryWrapper<TEExamBreachLog> teExamBreachLogQueryWrapper = new QueryWrapper<>();
            teExamBreachLogQueryWrapper.lambda().in(TEExamBreachLog::getExamRecordId, recordIdList)
                    .eq(TEExamBreachLog::getStatus, 0);
            teExamBreachLogList = this.list(teExamBreachLogQueryWrapper);
            teExamBreachLogList.forEach(s -> {
                s.setType(type.name());
                s.setDescription(String.valueOf(mapParameter.get("description")));
                s.setStatus(status);
                s.setUpdateId(tbUser.getId());
                ExamRecordCacheUtil.setBreachStatus(s.getExamRecordId(), 1);
                tOeExamRecordService.sendExamRecordDataSaveMq(s.getExamRecordId(), System.currentTimeMillis());
            });
        }
    }
}
